## auth finish

when 'authorizationStateReady'
  fetch_all_chats
  return
            
def fetch_all_chats
  offset_order = 2**63 - 1
  offset_chat_id = 0
  limit = 100

  loop do
    chat_ids = @client.fetch('@type' => 'getChats', 'offset_order' => offset_order, 'offset_chat_id' => offset_chat_id, 'limit' => limit).tap(&error_handler)['chat_ids']
    break if chat_ids.empty?
    last_chat = @client.fetch('@type' => 'getChat', 'chat_id' => chat_ids.last).tap(&error_handler)
    offset_chat_id, offset_order = last_chat.values_at('id', 'order')
  end
end

## get_chat
@client.on_ready do |client|
client.fetch('@type' => 'getChat', 'chat_id' => id)

## add_bot
@client.on_ready do |client|
chat = client.fetch('@type' => 'createPrivateChat', 'user_id' => bot_id)
client.fetch('@type' => 'sendBotStartMessage',
                                      'bot_user_id' => bot_id,
                                      'chat_id' => chat['id'])
end

## create_chat
"chat_create_supergroup '#{chatname}' '#{args[:sess][:chatDesc]}'",
"chat_export_invite_link #{chatname}",
"chat_add_user #{chatname} @#{args[:mmsg][:username]}",
"chat_add_user #{chatname} @#{args[:globals][:botUsername]}",
"chat_change_role #{chatname} @#{args[:globals][:botUsername]} editor"
        
@client.on_ready(timeout: 5) do |client|
  user_ids.each do |id|
    client.fetch('@type' => 'getUser', 'user_id' => id)
  end

  sleep 1

  chat = client.fetch('@type' => 'createNewBasicGroupChat',
                                      'title' => title,
                                      'user_ids' => user_ids)

  sleep 1

  client.fetch('@type' => 'toggleBasicGroupAdministrators',
                                'basic_group_id' => chat.dig('type', 'basic_group_id'),
                                'everyone_is_administrator' => false)
  chat
end


## get_user
@client.on_ready do |client|
    client.fetch('@type' => 'getUser', 'user_id' => user_id)
  end
end

## rename_chat
@client.on_ready do |client|
  client.fetch('@type' => 'setChatTitle',
                                'chat_id' => chat_id,
                                'title' => new_title)
end

## get_chat
@client.on_ready do |client|
  client.fetch('@type' => 'getChat', 'chat_id' => id)
end

## toggle_chat_admin
status =
  if admin
    { '@type' => 'chatMemberStatusAdministrator',
      'can_change_info' => true,
      'can_edit_messages' => true,
      'can_delete_messages' => true,
      'can_invite_users' => true,
      'can_restrict_members' => true,
      'can_pin_messages' => true,
      'can_promote_members' => true }
  else
    { '@type' => 'chatMemberStatusMember' }
  end
@client.on_ready do |client|
  client.fetch('@type' => 'getUser', 'user_id' => user_id)
  client.fetch('@type' => 'setChatMemberStatus',
                                'chat_id' => chat_id,
                                'user_id' => user_id,
                                'status' => status)
end

## delete_member
@client.fetch('@type' => 'setChatMemberStatus',
                              'chat_id' => chat_id,
                              'user_id' => user_id,
                              'status' => { '@type' => 'chatMemberStatusLeft' })

## close_chat
@client.on_ready do |client|
  me = client.fetch('@type' => 'getMe')
  bot_id = Setting.find_by(name: 'plugin_redmine_bots').value['telegram_bot_id'].to_i

  chat = client.fetch('@type' => 'getChat', 'chat_id' => chat_id)

  group_info = client.fetch('@type' => 'getBasicGroupFullInfo',
                                'basic_group_id' => chat.dig('type', 'basic_group_id')
  )
  return if group_info['@type'] == 'error'

  group_info['members'].map { |m| m['user_id'] }.each do |user_id|
    delete_member(chat_id, user_id) unless user_id.in?([me['id'], bot_id])
  end

  delete_member(chat_id, me['id'])
  delete_member(chat_id, bot_id)
end


## get_chat_link
@client.on_ready do |client|
  client.fetch('@type' => 'generateChatInviteLink', 'chat_id' => chat_id)
end


## error handling 
bot.api.send_message(message_params)
rescue BotKickedError
  logger.warn("Bot was kicked from chat. Chat Id: #{chat_id}, params: #{params.inspect}")
rescue FloodError => e
  logger.warn("Too many requests. Sleeping #{SLEEP_TIME} seconds...")
  sleep SLEEP_TIME
  (tries -= 1).zero? ? raise(e) : retry
end

SLEEP_TIME = 30

class BotKickedError
  def self.===(e)
    e.is_a?(Telegram::Bot::Exceptions::ResponseError) && e.message.include?('kicked')
  end
end

class FloodError
  def self.===(e)
    e.is_a?(Telegram::Bot::Exceptions::ResponseError) && e.message.include?('Too Many Requests')
  end
end
