########    #######    ########    #######    ########    ########
##     / / / /    License    \ \ \ \ 
##    Copyleft culture, Copyright (C) is prohibited here
##    This work is licensed under a CC BY-SA 4.0
##    Creative Commons Attribution-ShareAlike 4.0 License
##    Refer to the http://creativecommons.org/licenses/by-sa/4.0/
########    #######    ########    #######    ########    ########
##    / / / /    Code Climate    \ \ \ \ 
##    Language = ruby
##    Indent = space;    2 chars;
########    #######    ########    #######    ########    ########

module Dialog  
  class << self  
    
    
    def bot(args)
      args[:globals] = {}
      
      ### LANG
      i18n = Dialog::Lang.initial
      
      ###DRb
      DRb.start_service
      
      begin    
        Dialog::redisDBStore.call(['info', 'clients'])
        Dialog::redisDBCache.call(['info', 'clients']) 
        Dialog::redisDBStat.call(['info', 'clients'])                
      rescue
        Dialog.logger.error "No Redis connection!"
        sleep(2) 
        retry
      end
      
      ##Sidekiq
      Sidekiq.configure_client do |config|
        config.redis = { path: Dialog.config.db.redis_url, db: Dialog.config.db.redisDBBackground }
      end 
      
      #Dirs
      Dialog.directoriesMustExists
      
      Telegram::Bot::Client.run Dialog.config.secure.token do |bot|
        args[:bot]= bot; args[:bot].freeze
        
        #bot getMe
        Dialog.logger.info 'BotAPI getMe-request started'        
        me = bot.api.send(:getMe)
        Dialog.logger.info 'BotAPI getMe-request completed'     
        
        #bot globals
        args[:globals][:botUsername] = me["result"]["username"]
        args[:globals][:botFirstName] = me["result"]["first_name"]
        args[:globals][:botFirstNameArray] = me["result"]["first_name"].split("\s")
        args[:globals][:tdlibUser] = DRbObject.new_with_uri("drbunix:./_data/tdlib/#{Dialog.config.tdlib.instance}/drb.sock")        
        args[:globals].freeze
        
        DRb.start_service("drbunix:./_data/bot/#{Dialog.config.naming.instance}/drb.sock", bot)
        #DRb.thread.join
        
        #common queue
        args[:queue] = {}
        args[:queue][:threadCommon] = Concurrent::CachedThreadPool.new
        
        #Success
        Dialog.logger.info "Started: Bot named \"#{args[:globals][:botUsername]}\", instance=\"#{Dialog.config.naming.instance}\", interface=\"#{Dialog.config.naming.interface}\", ID=\"#{Dialog.config.naming.id}\", loglevel=\"#{Dialog.config.log.base}\""
        
        begin        
          bot.listen do |rmsg|
            args[:rmsg] = rmsg
            Dialog.clear(args) 
            Dialog.clearApi(args)
            args[:perf] = {init: Time.now}
              if args[:rmsg].respond_to?(:from)
                fromId = args[:rmsg][:from][:id]
                args[:queue][fromId] ||= {}
                args[:queue][fromId][:thread] = Concurrent::ImmediateExecutor.new unless args[:queue][fromId][:thread]
                args[:queue][fromId][:thread].post do
                  runMessage(args)
                end
              else
                Concurrent::Future.execute(executor: args[:queue][:threadCommon]) do
                  runMessage(args)
                end                
              end

          end
          rescue
#         rescue Exception => @e
          retry
#           Dialog.logger.error "ERROR - #{@e.inspect}\n\n Backtrace:\n #{@e.backtrace.join("\n\t")}\n \
#           MMSG:\n#{args[:mmsg]}\n" if @e             
        end        
      end
    end
    
    
    def runMessage(args)      
        begin
#           Dialog.logger.debug "######### - REDIS STAT:\n #{Dialog::redisDBCache.call(['client', 'list'])}\n\n"
          Dialog::Metamess.firstpoint(args)
          Dialog::Statistics.general(args)
          args[:perf][:statistics] = Time.now                               
          args[:perf][:fullend] = Time.now                     
          Dialog.logger.info("## PERF: #{perf(args[:perf]).join(", ")}")  
          
        rescue Exception => @e
          Dialog.logger.error "ERROR - #{@e.inspect}\n\n Backtrace:\n #{@e.backtrace.join("\n\t")}\n \
          MMSG:\n#{args[:mmsg]}\n
          RMSG:\n#{args[:rmsg].to_hash}\n" if @e
          args[:bot].api.send("sendMessage".to_sym, {chat_id: Dialog.config.monitoring.channel, disable_notification: true, text: @e}) unless Dialog.config.monitoring.channel.nil?
          raise
        end
    end

    
  end
end
