########    #######    ########    #######    ########    ########
##     / / / /    License    \ \ \ \ 
##    Copyleft culture, Copyright (C) is prohibited here
##    This work is licensed under a CC BY-SA 4.0
##    Creative Commons Attribution-ShareAlike 4.0 License
##    Refer to the http://creativecommons.org/licenses/by-sa/4.0/
########    #######    ########    #######    ########    ########
##    / / / /    Code Climate    \ \ \ \ 
##    Language = ruby
##    Indent = space;    2 chars;
########    #######    ########    #######    ########    ########

module Dialog
  extend Dry::Configurable
  
  require_relative './tooling/log.rb'
  def self.getDialogPath; return @dialogPath;  end
  def self.getDialog_rubyPath; return File.dirname(__FILE__);  end
  
  def self.checkEnv(env, val, err=[])
    if ENV[env].nil?
      case val
        when true;  res = true  
        when false; res = false && err << "There is no #{env} in envs"
        else        res = val 
      end
    else res = ENV[env]; end
    return res
  end    
  
  err = []
  app = 'Dialog'  
  
  setting :tdlib do
    setting :id,     Dialog.checkEnv('ApiID',   false, err)
    setting :hash_s, Dialog.checkEnv('ApiHash', false, err)
    setting :instance, Dialog.checkEnv('TdlibInstance', false, err)    
  end
  
  _scope = Dialog.checkEnv('runScope', false, err)
  _mode =  Dialog.checkEnv('runMode',  false, err)
  _instance = Dialog.checkEnv('runInstance',  false, err)
  setting :run do
    setting :scope, _scope 
    setting :mode,  _mode
  end  
  
  setting :naming do
    setting :app, app
  end

  _interface = Dialog.checkEnv('Interface', 'Demo')
  unless _scope == 'deploy' or _mode == 'background'
    token = Dialog.checkEnv('BotToken',  false, err)    
    
    setting :naming do    
      setting :app, app
      setting :interface, _interface      
      unless _instance == false
        setting :instance, _instance
      else
        err << "No instance!"        
      end
      setting :id, token.split(':')[0].to_i unless token == false
    end  
    
    setting :dialect do
      setting :sqlite, Dialog.checkEnv('DialectSqlite', "#{Dialog.getDialogPath}/dialect/dialect.sqlite")
      setting :json,   Dialog.checkEnv('DialectJSON',   "#{Dialog.getDialogPath}/dialect/#{_interface}.json")      
    end    
    
    setting :secure do
      setting :token,  token
    end     
  end
  
  setting :db do
    setting :redis_url,  "./_data/redis/redis.sock"
    setting :redisDBBackground, 0    
    setting :redisDBStore, 1
    setting :redisDBStat, 2
    setting :redisDBCache, 3    
    
  end     

  setting :proxy do
    setting :type,  Dialog.checkEnv('BotProxyType',  false, err)
    setting :host,  Dialog.checkEnv('BotProxyHost',  false, err)
    setting :port,  Dialog.checkEnv('BotProxyPort',  false, err)
    setting :login, Dialog.checkEnv('BotProxyLogin', false, err)
    setting :pass,  Dialog.checkEnv('BotProxyPass',  false, err)
  end
  
  setting :path do
    setting :dialog_ruby,   "#{Dialog.getDialog_rubyPath}/../"
    setting :dialog_rubyLib,"#{Dialog.getDialog_rubyPath}"
    setting :dialog,        "#{Dialog.getDialogPath}"        
    setting :dialogData,    "#{Dialog.getDialogPath}/_data"    
    setting :dialogRender,  "#{Dialog.getDialogPath}/_render/#{_instance}"
    setting :dialogTmpfiles,"#{Dialog.getDialogPath}/_render/#{_instance}/tmpfiles"        
  end 
  
  setting :log do
    setting :base,  Dialog.checkEnv('LogLevel', 'info')
    setting :tdlib, Dialog.checkEnv('LogLevel', 1)
  end  
  
  setting :monitoring do
    setting :channel,  Dialog.checkEnv('MonitoringChannel', true)
  end    
  
  setting :host do
    setting :apiUri,  "https://api.telegram.org"
    botInstanceHostName = Dialog.checkEnv('BotInstanceHostName', true)
    setting :instance,  botInstanceHostName
    setting :web_hook_url, "https://#{botInstanceHostName}/telegram/api/web_hook"    
  end 
  
  if _scope == false 
    case _mode
    when false 
      FileUtils.cp "#{Dialog.getDialog_rubyPath}/../exec/dialog.sh", "#{Dialog.getDialogPath}/" if not File.file?("#{Dialog.getDialogPath}/dialog.sh")
      Dialog.logger.info "dialog.sh is copied to your dialog folder, please use it for correct ENV exports,\nbash ./dialog.sh"
      Dialog.logger.info "Deploying dialog base"   
      deploy = true
    when 'dev'
      deploy = true
    end
    Dialog::Triggers.trigger_deploy if deploy == true
  end
  
  unless err.empty?
    err << "EXITING!"
    Dialog.logger.debug "ERRORS: #{err}"
    exit(false)
  end
end
