########    #######    ########    #######    ########    ########
##     / / / /    License    \ \ \ \ 
##    Copyleft culture, Copyright (C) is prohibited here
##    This work is licensed under a CC BY-SA 4.0
##    Creative Commons Attribution-ShareAlike 4.0 License
##    Refer to the http://creativecommons.org/licenses/by-sa/4.0/
########    #######    ########    #######    ########    ########
##    / / / /    Code Climate    \ \ \ \ 
##    Language = ruby
##    Indent = space;    2 chars;
########    #######    ########    #######    ########    ########
module Dialog  
  module TDLib
    class << self

    def tdlibAuth(mode, tgSess)
      case mode
      when 'bot'
        bot_auth(tgSess)
      when 'user'
        user_auth(tgSess)
      end
    end


    def proxy
      proxy = {
        '@type' => 'addProxy',
        'server'   => Dialog.config.proxy.host,
        'port'     => Dialog.config.proxy.port,
        'enable'   => true,
        'type' => {
          '@type'    => 'proxyTypeSocks5', 
          'username' => Dialog.config.proxy.login,
          'password' => Dialog.config.proxy.pass
        }
      }    
      return proxy
    end
    
    def bot_auth(tgSess)
      state = nil
      
      tgSess.on('updateAuthorizationState') do |update|
        next unless update.dig('authorization_state', '@type') == 'authorizationStateWaitPhoneNumber'
        state = :wait_token
      end  
      
      tgSess.on('updateAuthorizationState') do |update|
        next unless update.dig('authorization_state', '@type') == 'authorizationStateReady'
        state = :ready
      end
      
      loop do
        case state
        when :wait_token
          Dialog.logger.info 'Please enter bot token:'
          token = STDIN.gets.strip      
          params = {
            '@type' => 'checkAuthenticationBotToken',
            'token' => token
          }
          tgSess.broadcast_and_receive(params).tap(&error_handler)
        
        when :ready
          @me = tgSess.broadcast_and_receive('@type' => 'getMe')
          break    

        end 
      end
      close(tgSess)
    end



    def user_auth(tgSess)
      state = nil
      
      tgSess.broadcast(proxy)  

      tgSess.on('updateAuthorizationState') do |update|
        next unless update.dig('authorization_state', '@type') == 'authorizationStateWaitPhoneNumber'
        state = :wait_phone
      end
      
      tgSess.on('updateAuthorizationState') do |update|
        next unless update.dig('authorization_state', '@type') == 'authorizationStateWaitPassword'
        state = :wait_password
      end  

      tgSess.on('updateAuthorizationState') do |update|
        next unless update.dig('authorization_state', '@type') == 'authorizationStateWaitCode'
        state = :wait_code
      end

      tgSess.on('updateAuthorizationState') do |update|
        next unless update.dig('authorization_state', '@type') == 'authorizationStateReady'
        state = :ready
      end

      loop do
        p state
        case state
        when :wait_phone
          Dialog.logger.error 'Please enter phone number:'
          phone = STDIN.gets.strip
          params = {
            '@type' => 'setAuthenticationPhoneNumber',
            'phone_number' => phone
          }
          tgSess.broadcast_and_receive(params).tap(&error_handler)
          
        when :wait_code
          Dialog.logger.error 'Please enter code from SMS:'
          code = STDIN.gets.strip
          params = {
            '@type' => 'checkAuthenticationCode',
            'code' => code
          }
          tgSess.broadcast_and_receive(params).tap(&error_handler)
          
        when :wait_password
          Dialog.logger.error 'Please enter password:'
          password = STDIN.gets.strip
          params = {
            '@type' => 'checkAuthenticationPassword',
            'password' => password
          }
          tgSess.broadcast_and_receive(params).tap(&error_handler) 
        when :ready
          @me = tgSess.broadcast_and_receive('@type' => 'getMe')
          Dialog.logger.error 'TDLib cli init success!'          
          break
        end
      end

      close(tgSess)
    end

    end
  end
end
