########    #######    ########    #######    ########    ########
##     / / / /    License    \ \ \ \ 
##    Copyleft culture, Copyright (C) is prohibited here
##    This work is licensed under a CC BY-SA 4.0
##    Creative Commons Attribution-ShareAlike 4.0 License
##    Refer to the http://creativecommons.org/licenses/by-sa/4.0/
########    #######    ########    #######    ########    ########
##    / / / /    Code Climate    \ \ \ \ 
##    Language = ruby
##    Indent = space;    2 chars;
########    #######    ########    #######    ########    ########

module Dialog
  module Triggers
    class << self     
    
      def trigger_deploy
        require 'erb'      
        mode = Dialog.config.run.mode
        Dialog.logger.debug "Using mode '#{mode}'"      
        FileUtils.mkdir_p "#{Dir.home}/.config/systemd/user"
        deploy_redis_run unless File.file?("#{Dialog.config.path.dialogData}/redis/dump.rdb")
        deploy_background_systemd
        deploy_binaries_notice
        deploy_bot_systemd
        deploy_tdlib_systemd
        deploy_bot_dev_systemd if Dialog.config.run.mode == 'dev' 
        Open3.popen3('/usr/bin/systemctl', '--user', 'daemon-reload')
        deploy_gems unless File.file?("#{Dir.home}/.config/systemd/user/dialog_bot_dev@.service")
      end
      
      
      def deploy_redis_run
        if File.file?('/usr/sbin/redis-server')
          redisDir = "#{Dialog.config.path.dialogData}/redis"
          FileUtils.mkdir_p redisDir
          erbGen('systemd/dialog_redis.service', "#{Dir.home}/.config/systemd/user/dialog_redis.service")
          erbGen('conf/redis_local.conf', "#{redisDir}/redis_local.conf")
          erbGen('conf/redis_global.conf', "#{redisDir}/redis_global.conf")
          systemd('dialog_redis', 'enstart')
          Dialog.logger.info "Redis deployment OK!"          
        else
          Dialog.logger.error "Please install Redis binary"
          Dialog.logger.info "openSUSE: zypper --no-gpg-checks --non-interactive in redis"
          exit(false)
        end
      end
      
      
      def deploy_bot_systemd
        erbGen('systemd/dialog_runMode@.service', "#{Dir.home}/.config/systemd/user/dialog_bot@.service", {:mode => 'bot'})
        erbGen('systemd/dialog_runner@.path',    "#{Dir.home}/.config/systemd/user/dialog_bot_pre@.path", {:desc => "dialog_bot_pre runner", :path => "/tmp/in4/dialog_touch_%i"})            
        erbGen('systemd/dialog_runMode@.service', "#{Dir.home}/.config/systemd/user/dialog_bot_pre@.service",  {:mode => 'check', :env => 'bot', :systemdChain => "dialog_bot"})  
        Dialog.logger.info "Bot systemd services deployed OK!"                  
      end      

      
      def deploy_tdlib_systemd
        erbGen('systemd/dialog_runMode@.service', "#{Dir.home}/.config/systemd/user/dialog_tdlib@.service", {:mode => 'tdlib'})   
        Dialog.logger.info "TDLib systemd services deployed OK!"
      end
      
      
      def deploy_background_systemd
        erbGen('systemd/dialog_background.service', "#{Dir.home}/.config/systemd/user/dialog_background.service")
        systemd('dialog_background', 'enstart')        
        Dialog.logger.info "Background systemd services deployed OK!"
      end  
      
      
      def deploy_bot_dev_systemd
        erbGen('systemd/dialog_runner@.path',    "#{Dir.home}/.config/systemd/user/dialog_bot_dev@.path", {:desc => 'dialog_bot_dev runner', :path => "#{Dialog.config.path.dialog}/dialect/dialect.sqlite"})                  
        erbGen('systemd/dialog_runMode@.service', "#{Dir.home}/.config/systemd/user/dialog_bot_dev@.service", {:mode => 'dialect', :env => 'bot', :systemdChain => 'dialog_bot'}) 
        Dialog.logger.info "Bot development systemd services deployed OK!"        
      end         

      
      def deploy_binaries_notice
        Dialog.logger.warn "Please install neccessary binaries:"
        Dialog.logger.info "openSUSE: zypper ar -cf https://download.opensuse.org/repositories/home:/in4ops:/4wrap/openSUSE_Leap_15.0/home:in4ops:4wrap.repo"
        #will be 'home:in4ops:Innosense:libs'
        Dialog.logger.info "openSUSE: zypper --no-gpg-checks --non-interactive in zbar tdlib" 
        Dialog.logger.info "sudo loginctl enable-linger $USER"
      end
      
      
      def deploy_gems
        stderr, status = Open3.capture3("gem specific_install -u -l https://gitlab.com/Innosense/libs/dialog_bot_ruby.git", :stdin_data => 'stdin')
        Dialog.logger.info stderr
        stderr, status = Open3.capture3("gem specific_install -u -l https://gitlab.com/Innosense/MVP/dialect_lookup.git", :stdin_data => 'stdin')
        Dialog.logger.info stderr
        stderr, status = Open3.capture3("gem specific_install -u -l https://gitlab.com/Innosense/MVP/dialog_ruby.git", :stdin_data => 'stdin')
        Dialog.logger.info stderr        
        Dialog.logger.info "Gems deployment OK!"                  
      end
      

      def deploy_gitsync
      end  
      

    end
  end
end
